import TransactionService from './transaction.service'

export default {
  getOverview(config) {
    const ta = [
      { name: 'nd.bcd.list', old: { is_own: true, sorting_params_list: ['name'] } },
      { name: 'org.unit.list', join: { 0: 'api_fkey_nd_bcd_ou' } },
      { name: 'nd.ip_subnet.list', join: { 0: 'api_fkey_nd_ip_subnet_bcd' } },
      { name: 'nd.vlan.list', join: { 0: 'api_fkey_nd_vlan_bcd' } }
    ]
    return TransactionService.execute(config, ta)
  },
  getOverviewV4Only(config) {
    const ta = [
      { name: 'nd.ip_subnet.list', old: { is_own: true, type: '4' } },
      { name: 'nd.bcd.list', join: { 0: 'api_fkey_nd_ip_subnet_bcd' } },
      { name: 'org.unit.list', join: { 1: 'api_fkey_nd_bcd_ou' } },
      { name: 'nd.vlan.list', join: { 1: 'api_fkey_nd_vlan_bcd' } }
    ]
    return TransactionService.execute(config, ta)
  },
  getDetail(config, name) {
    const ta = [
      { name: 'nd.bcd.list', old: { name_list: [name] } },
      { name: 'org.unit.list', join: { 0: 'api_fkey_nd_bcd_ou' } },
      { name: 'nd.ip_subnet.list', join: { 0: 'api_fkey_nd_ip_subnet_bcd' } },
      {
        name: 'dns.ip_addr.list',
        old: { is_reserved: true, sorting_params_list: ['value'] },
        join: { 2: 'api_fkey_dns_ip_addr_subnet' }
      },
      { name: 'nd.bcd2group.list', join: { 0: 'api_fkey_nd_bcd2group_bcd' } },
      { name: 'cntl.group.list', join: { 4: 'api_fkey_nd_bcd2group_grp' } },
      { name: 'cntl.mgr2group.list', join: { 5: 'api_fkey_cntl_mgr2group_grp' } },
      { name: 'cntl.mgr.list', join: { 6: 'api_fkey_cntl_mgr2group_mgr' } },
      { name: 'nd.ip_route.list', join: { 2: 'api_fkey_nd_ip_route_subnet' } },
      { name: 'dns.ip_addr.list', join: { 2: 'api_fkey_dns_ip_addr_subnet' } },
      { name: 'nd.l_port.list', old: { adm_state: 1 }, join: { 8: 'api_fkey_nd_ip_route_destlp' } },
      { name: 'nd.ip_intf.list', join: { 9: 'api_fkey_nd_ip_intf_addr', 10: 'api_fkey_nd_ip_intf_port' } },
      { name: 'dns.fqdn2group.list', join: { 5: 'api_fkey_dns_fqdn2group_grp' } },
      {
        name: 'org.unit.list',
        old_ref: { short_name: { idx: 1, param: 'short_name' } },
        old: { do_rootpath: true }
      },
      { name: 'cntl.mgr2ou.list', join: { 13: 'api_fkey_cntl_mgr2ou_ou' } },
      { name: 'cntl.mgr.list', join: { 14: 'api_fkey_cntl_mgr2ou_mgr' } },
      { name: 'nd.vlan.list', join: { 0: 'api_fkey_nd_vlan_bcd' } }
    ]
    return TransactionService.execute(config, ta)
  },
  getRecords(config, bcd) {
    const ta = [
      {
        name: 'dns.record.list',
        old: {
          sorting_params_list: ['type', 'target_ipaddr', 'data', 'fqdn'],
          bcd_list: [bcd]
        }
      },
      { name: 'dns.fqdn.list', join: { 0: 'api_fkey_dns_record_fqdn' } },
      { name: 'dnscfg.record_inttype.list', old: { is_own: true, sorting_params_list: ['record_type'] } },
      { name: 'dnscfg.record_type.list', old: { is_internal: false } },
      { name: 'dnscfg.fqdn_type.list', old: { sorting_params_list: ['position'] } }
    ]
    return TransactionService.execute(config, ta)
  }
}
