import TransactionService from './transaction.service'
import apiutil from '@/util/apiutil'

function checkRegex(term) {
  if (term.startsWith('s/')) {
    return term.slice(2)
  }
  return '(?i).*' + term + '.*'
}

export default {
  searchBCDRegex(config, term, own) {
    const ta = [
      {
        name: 'nd.bcd.list',
        old: apiutil.denullify_dict({
          name_regexp: checkRegex(term),
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['name']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchRecordRegex(config, term, own) {
    const ta = [
      {
        name: 'dns.record.list',
        old: apiutil.denullify_dict({
          fqdn_regexp: checkRegex(term),
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['fqdn']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchRRDataRegex(config, term, own) {
    const ta = [
      {
        name: 'dns.record.list',
        old: apiutil.denullify_dict({
          target_data_unref_regexp: '(?i).*' + term + '.*',
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['target_data_unref']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchRRByIP(config, ip, own) {
    const ta = [
      {
        name: 'dns.record.list',
        old: apiutil.denullify_dict({
          target_ipaddr_cidr: ip,
          is_own: own,
          fetch_limit: 20,
          sorting_params_list: ['target_ipaddr']
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchVlan(config, term) {
    const ta = [
      {
        name: 'nd.vlan.list',
        old: apiutil.denullify_dict({
          id: term,
          fetch_limit: 20
        })
      }
    ]
    return TransactionService.execute(config, ta)
  },
  searchBCDSubnet(config, term, own) {
    const ta = [
      {
        name: 'nd.ip_subnet.list',
        old: apiutil.denullify_dict({
          cidr: term,
          is_own: own,
          fetch_limit: 20,
          cidr_operator: 'smallest_cts'
        })
      }
    ]
    return TransactionService.execute(config, ta)
  }
}
