import Vue from 'vue'
import App from './App.vue'
import router from './router'
import Vuex from 'vuex'
import Axios from 'axios'
import SystemInfoService from './api-services/system_info.service'
import BootstrapVue from 'bootstrap-vue'
import titleMixin from './mixins/title_mixin'
import isMobileMixin from './mixins/is_mobile_mixin'

import './icons'

import sharedMutations from 'vuex-shared-mutations'
import vuexUndoRedo from './mixins/vuex-undo-redo'

import APIUtil from './util/apiutil'
import WAPITransactionSerice from './api-services/transaction.service'
import LoginService from '@/api-services/login.service'
import {EventBus} from '@/eventbus'

Vue.use(Vuex)
Vue.mixin(titleMixin)
Vue.mixin(isMobileMixin)
Vue.use(BootstrapVue)
Vue.use(vuexUndoRedo, { emptyStateMutation: 'emptyState' })

async function init() {
  let session_info = null
  let stored_token = null
  try {
    session_info = (await SystemInfoService.getSessionInfo()).data
    stored_token = session_info.token
  } catch (e) {
    window.console.error('failed receiving session info')
  }
  const netdb_axios_config_tmp = {
    headers: {
      Authorization: 'Bearer ' + (stored_token ? stored_token.token : 'INVALID'),
      'Content-Type': 'application/json'
    }
  }
  window.console.debug(netdb_axios_config_tmp)

  // TODO: Refactor prototype definitions into their own file
  // eslint-disable-next-line no-extend-native
  Array.prototype.move = function (from, to) {
    this.splice(to, 0, this.splice(from, 1)[0])
    return this
  }
  if (!('toJSON' in Error.prototype)) {
    // eslint-disable-next-line no-extend-native
    Object.defineProperty(Error.prototype, 'toJSON', {
      value: function () {
        var alt = {}

        Object.getOwnPropertyNames(this).forEach(function (key) {
          alt[key] = this[key]
        }, this)

        return alt
      },
      configurable: true,
      writable: true
    })
  }

  const store = new Vuex.Store({
    state: {
      dark_mode: JSON.parse(window.localStorage.getItem('dark_mode')) || false,
      user: session_info ? session_info.user : null,
      token: session_info ? session_info.token : null,
      ta_list: JSON.parse(window.localStorage.getItem('ta_list')),
      netdb_axios_config: netdb_axios_config_tmp,
      done: JSON.parse(window.localStorage.getItem('done')) || [],
      undone: JSON.parse(window.localStorage.getItem('undone')) || [],
      undo_redo_new_mutaion: JSON.parse(window.localStorage.getItem('undo_redo_new_mutation')) || true,
      transaction_result: JSON.parse(window.localStorage.getItem('transaction_result')) || null,
      executing_transaction: JSON.parse(window.localStorage.getItem('executing_transaction')) || false,
      show_sidebar: JSON.parse(window.localStorage.getItem('show_sidebar')) || false,
      reload_count: 0,
      db_editor_cache: JSON.parse(window.localStorage.getItem('db_editor_cache')) || {}
    },
    mutations: {
      toggleDarkMode(state) {
        state.dark_mode = !state.dark_mode
        window.localStorage.setItem('dark_mode', JSON.stringify(state.dark_mode))
      },
      emptyState(state) {
        state.ta_list = null
        window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
      },
      setUndoRedoNewMutation(state, payload) {
        state.undo_redo_new_mutaion = payload
        window.localStorage.setItem('undo_redo_new_mutation', JSON.stringify(state.undo_redo_new_mutaion))
      },
      popUndoRedoDone(state) {
        state.done.pop()
        window.localStorage.setItem('done', JSON.stringify(state.done))
      },
      popUndoRedoUndone(state) {
        state.undone.pop()
        window.localStorage.setItem('undone', JSON.stringify(state.undone))
      },
      pushUndoRedoDone(state, payload) {
        state.done.push(payload)
        window.localStorage.setItem('done', JSON.stringify(state.done))
      },
      pushUndoRedoUndone(state, payload) {
        state.undone.push(payload)
        window.localStorage.setItem('undone', JSON.stringify(state.undone))
      },
      clearUndoRedoDone(state) {
        state.done = []
        window.localStorage.setItem('done', JSON.stringify(state.done))
      },
      clearUndoRedoUndone(state) {
        state.undone = []
        window.localStorage.setItem('undone', JSON.stringify(state.undone))
      },
      moveTaElements(state, payload) {
        state.ta_list.move(payload.from, payload.to)
        window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
      },
      login(state, payload) {
        const user = payload.user
        const token = payload.token
        state.user = user
        state.token = token
        state.netdb_axios_config = {
          headers: {
            Authorization: 'Bearer ' + token.token,
            'Content-Type': 'application/json'
          }
        }
      },
      logout(state) {
        state.user = null
        state.token = null
        state.ta_list = []
        state.undo_redo_new_mutaion = true
        state.undone = []
        state.done = []
        state.transaction_result = null
        state.executing_transaction = false
        state.show_sidebar = false
        window.localStorage.removeItem('done')
        window.localStorage.removeItem('undone')
        window.localStorage.removeItem('undo_redo_new_mutation')
        window.localStorage.removeItem('ta_list')
        window.localStorage.removeItem('transaction_result')
        window.localStorage.removeItem('executing_transaction')
        window.localStorage.removeItem('show_sidebar')
        window.localStorage.removeItem('db_editor_cache')
        state.netdb_axios_config = {
          headers: {
            Authorization: null,
            'Content-Type': 'application/json'
          }
        }
      },
      removeTransactionElement(state, payload) {
        for (let i = 0; i < state.ta_list.length; i++) {
          if (state.ta_list[i].uuid === payload.uuid) {
            state.ta_list.splice(i, 1)
            break
          }
        }
        window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
      },
      replaceTransactionList(state, new_list) {
        state.ta_list = new_list
        window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
      },
      addTransactionElement(state, payload) {
        if (state.ta_list == null) {
          state.ta_list = []
        }
        let update = false
        for (let i = 0; i < state.ta_list.length; i++) {
          if (state.ta_list[i].uuid === payload.old_uuid) {
            update = true
            Vue.set(state.ta_list, i, payload)
            break
          }
        }
        if (!update) {
          state.ta_list.push(payload)
        }
        window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
      },
      emptyTransactionList(state) {
        state.ta_list = []
        window.localStorage.setItem('ta_list', JSON.stringify(state.ta_list))
      },
      async testTransaction(state) {
        this.commit('setTransactionBusy', true)

        const api_list = APIUtil.buildAPITaFromTaObjectArray(state.ta_list)
        try {
          const res = await WAPITransactionSerice.executeDry(state.netdb_axios_config, api_list)
          window.console.debug(res)
          state.transaction_result = {
            type: 'test-success'
          }
        } catch (e) {
          window.console.debug(e.response)
          state.transaction_result = {
            type: 'error',
            error: e.response.data.exception,
            uuid: state.ta_list[APIUtil.getAPIErrorIndexFromDBException(e.response.data.exception)].uuid
          }
        } finally {
          this.commit('setTransactionBusy', false)
          window.localStorage.setItem('transaction_result', JSON.stringify(state.transaction_result))
        }
      },
      setTransactionResult(state, result) {
        state.transaction_result = result
        window.localStorage.setItem('transaction_result', JSON.stringify(state.transaction_result))
      },
      showSidebar(state, show) {
        state.show_sidebar = show
        window.localStorage.setItem('show_sidebar', show)
      },
      setTransactionBusy(state, busy) {
        state.executing_transaction = busy
        window.localStorage.setItem('executing_transaction', busy)
      },
      removeTransactionResult(state) {
        state.transaction_result = null
        window.localStorage.removeItem('transaction_result')
      },
      reloadRouterComp(state) {
        state.reload_count++
      },
      cacheDBEditorData(state, data) {
        state.db_editor_cache[data.object_fq_name] = data
        window.localStorage.setItem('db_editor_cache', JSON.stringify(state.db_editor_cache))
      }
    },
    actions: {
      async executeTransaction({ commit, state }) {
        commit('setTransactionBusy', true)

        const api_list = APIUtil.buildAPITaFromTaObjectArray(state.ta_list)
        try {
          const res = await WAPITransactionSerice.execute(state.netdb_axios_config, api_list)
          commit('setTransactionResult', {
            type: 'success',
            result: res.data,
            ta_list: state.ta_list
          })
          commit('emptyTransactionList')
          commit('clearUndoRedoDone')
          commit('clearUndoRedoUndone')
          commit('setUndoRedoNewMutation', true)
        } catch (e) {
          window.console.debug(e.response)
          commit('setTransactionResult', {
            type: 'error',
            error: 'exception' in e.response.data ? e.response.data.exception : e.response.data,
            uuid: 'exception' in e.response.data ? state.ta_list[APIUtil.getAPIErrorIndexFromDBException(e.response.data.exception)].uuid : null
          })
        } finally {
          commit('setTransactionBusy', false)
        }
      }
    },
    plugins: [
      sharedMutations(
        {
          predicate:
            [
              'pushUndoRedoUndone',
              'popUndoRedoUndone',
              'clearUndoRedoUndone',
              'setUndoRedoNewMutation',
              'clearUndoRedoDone',
              'popUndoRedoDone',
              'emptyState',
              'login',
              'logout',
              'addTransactionElement',
              'removeTransactionElement',
              'emptyTransactionList',
              'removeTransactionResult',
              'setTransactionBusy',
              'moveTaElements',
              'replaceTransactionList',
              'setTransactionResult'
            ]
        }
      )
    ]
  })
  // Undo-Redo-watched mutations
  const watchedMutations = ['addTransactionElement', 'removeTransactionElement', 'emptyTransactionList', 'moveTaElements', 'replaceTransactionList']

  Vue.config.errorHandler = function (err, vm, info) {
    // handle error
    // `info` is a Vue-specific error info, e.g. which lifecycle hook
    // the error was found in. Only available in 2.2.0+
    // alert(err, info)
    window.console.debug('vue-error handler triggered')
    window.console.log(err, info)
    EventBus.$emit('error', err)
  }
  window.onerror = function (message, source, lineno, colno, error) {
    window.console.debug('window-error handler triggered')
    window.console.log(error)
    EventBus.$emit('error', error)
  }
  window.addEventListener('unhandledrejection', function (event) {
    window.console.debug('unhanldedrejection handler triggered')
    EventBus.$emit('error', event.reason)
  })
  Axios.interceptors.response.use(
    function (response) {
      return response
    },
    async function (error) {
      // handle error
      if (error.response) {
        var stat = error.response.status
        switch (stat) {
          case 401:
            await LoginService.logout(null)
            store.commit('logout')
            if (router.currentRoute.path !== '/login') {
              router.push('/login')
            }
            break
          case 403:
            await LoginService.logout(null)
            store.commit('logout')
            if (router.currentRoute.path !== '/login') {
              router.push('/login')
            }
            break
          default:
            break
        }
      } else {
        switch (stat) {
          case 401:
            await LoginService.logout(null)
            store.commit('logout')
            router.push('/login')
            break
          default:
            alert(error)
            break
        }
      }
      return Promise.reject(error)
    })
  Vue.config.productionTip = false
  Axios.defaults.baseURL = '/'

  SystemInfoService.getAll().then((sysinfo) => {
    Vue.prototype.$sysinfo = sysinfo.data
    const mods_by_name = {}
    for (var i = 0; i < sysinfo.data.mods.length; i++) {
      mods_by_name[sysinfo.data.mods[i].name] = sysinfo.data.mods[i]
    }
    Vue.prototype.$sysinfo_mods_by_name = mods_by_name
    Vue.prototype.$sysinfo_js_mods = sysinfo.data.js_mods
    const v = new Vue({
      router,
      store,
      render: h => h(App),
      beforeCreate: function () {
      }
    }).$mount('#app')
    if (v.$store) {
      v.$store.subscribe((mutation, state) => {
        if (watchedMutations.indexOf(mutation.type) >= 0) {
          v.$store.commit('pushUndoRedoDone', mutation)
          if (state.undo_redo_new_mutaion) {
            window.console.debug('New mutation')
            v.$store.commit('clearUndoRedoUndone')
          }
        }
      })
    }
  })
}

init()
