export default {
  ipv6_to_ints (v6) {
    const doubleBytes = v6.split(':')
    const res = []
    for (let i = 0; i < 4; i++) {
      res.push(parseInt('0x' + doubleBytes[2 * i] + doubleBytes[2 * i + 1], 16))
    }
    return res
  },
  ip_to_int (ip) {
    let byteString = '0x'
    const bytes = ip.split('.')
    for (let i = 0; i < 4; i++) {
      let res = Number(bytes[i]).toString(16)
      if (res.length === 1) {
        res = '0' + res
      }
      byteString += res
    }
    return parseInt(byteString, 16)
  },
  is_ip_v4 (ip) {
    return ip.split('.').length === 4
  },
  ip_num_addr (net) {
    return Math.pow(2, 32 - Number(net.split('/')[1]))
  },
  ipv6_num_addr (net6) {
    return Math.pow(2, 128 - Number(net6.split('/')[1]))
  },
  ip_net_get_last (net) {
    return this.int_to_ip(this.ip_to_int(net.split('/')[0]) + this.ip_num_addr(net) - 1)
  },
  ip_net_get_first (net) {
    return net.split('/')[0]
  },
  ip_net_get_mask (net) {
    const s = parseInt(net.split('/')[1])
    return this.int_to_ip(parseInt('1'.repeat(s) + '0'.repeat(32 - s), 2))
  },
  int_to_ip (ip) {
    let bytestring = ip.toString(16)
    while (bytestring.length < 8) {
      bytestring = '0' + bytestring
    }
    const bytes = []
    for (let i = 0; i < 8; i++) {
      bytes.push(parseInt(bytestring.substr(i, 2), 16))
      i++
    }
    return bytes.join('.')
  },
  diff_ip4 (a, b) {
    const a_ip = this.ip_to_int(b)
    const b_ip = this.ip_to_int(a)
    if (a_ip > b_ip) {
      return a_ip - b_ip
    }
    return b_ip - a_ip
  },
  compare (a, b) {
    if (this.is_ip_v4(a) && this.is_ip_v4(b)) {
      return this.ip_to_int(a) - this.ip_to_int(b)
    }
    const ip6a = this.ipv6_to_ints(a)
    const ip6b = this.ipv6_to_ints(b)
    for (let i = 0; i < 4; i++) {
      const r = ip6a[i] - ip6b[i]
      if (r === 0) {
        continue
      }
      return r
    }
    return 0
  }
}
