import { v4 as uuidv4 } from 'uuid'
import ObjectTypeService from '@/api-services.gen/wapi.object_type'
import FunctionService from '@/api-services.gen/wapi.function'
import ApiUtil from '@/util/apiutil'

export default {
  function2variant (function_name, outline = false) {
    const prefix = outline ? 'outline-' : ''
    switch (function_name) {
      case 'create':
        return prefix + 'success'
      case 'delete':
        return prefix + 'danger'
      case 'update':
      case 'bulk_update':
        return prefix + 'primary'
      default:
        return prefix + 'default'
    }
  },
  function2text (function_name) {
    switch (function_name) {
      case 'create':
        return 'anlegen'
      case 'delete':
        return 'löschen'
      case 'update':
        return 'bearbeiten'
      case 'bulk_update':
        return 'bearbeiten (bulk)'
      default:
        return function_name
    }
  },
  nullify (data) {
    if (data === '__NULL') {
      return null
    }
    return data
  },
  filter_unchanged_data (d1, d2) {
    const self = this
    return Object.keys(d1).reduce(function (filtered, key) {
      if (!(key in d2) || d1[key] !== d2[key]) filtered[key] = self.nullify(d1[key])
      return filtered
    }, {})
  },
  filter_old_data (params_list, old_data) {
    return Object.keys(old_data).reduce(function (filtered, key) {
      if (key in params_list && 'old' in params_list[key]) filtered[key] = old_data[key]
      return filtered
    }, {})
  },
  remove_empty_data (params_list, data) {
    return Object.keys(data).reduce(function (filtered, key) {
      if (!(data[key] === '')) filtered[key] = data[key]
      return filtered
    }, {})
  },
  generateDeleteElement (object_fq_name, params_list, old_data, title) {
    const ta = {}
    ta.parameters = {
      new: {},
      old: Object.keys(old_data).reduce(function (filtered, key) {
        if (params_list.old.includes(key)) filtered[key] = old_data[key]
        return filtered
      }, {})
    }
    ta.uuid = uuidv4()
    ta.object_fq_name = object_fq_name
    ta.object_function = 'delete'
    ta.object_functions = null
    ta.object_title = title
    return ta
  },
  async validateObjectFQName(vm, object_fq_name) {
    const parts = object_fq_name.split('.')
    const system = parts[0]
    const name = parts[1]
    const object_result = ObjectTypeService.list(vm.$store.state.netdb_axios_config, {
      name_list: [name],
      system_list: [system]
    })
    try {
      let object_definition = await object_result
      object_definition = object_definition.data[0]
      if (object_definition.length !== 1) {
        return false
      }
      const obj_functions = await FunctionService.list(vm.$store.state.netdb_axios_config, {
        object_type_list: [name],
        system_list: [system]
      })
      window.console.debug(ApiUtil.dict_by_value_of_array(obj_functions.data[0], 'name'))
      return {
        object_definition: object_definition[0],
        object_functions: ApiUtil.dict_by_value_of_array(obj_functions.data[0], 'name')
      }
    } catch (e) {
      return false
    }
  },
  async getObjectMetadata(vm, object_fq_name) {
    const res = {}
    if (object_fq_name in vm.$store.state.db_editor_cache) {
      window.console.debug('DBEditor cache hit')
      const entry = vm.$store.state.db_editor_cache[object_fq_name]
      res.object_definition = entry.object_definition
      res.object_functions = entry.object_functions
      res.object_definition_referencing_by_attribute = entry.object_definition_referencing_by_attribute
      res.object_definition_reference_names_by_attribute = entry.object_definition_reference_names_by_attribute
      return res
    }
    const data = await this.validateObjectFQName(vm, object_fq_name)
    if (data === false) {
      return null
    } else {
      res.object_definition = data.object_definition
      res.object_definition_referencing_by_attribute = {}
      res.object_definition_reference_names_by_attribute = {}
      for (const ref in res.object_definition.referencing) {
        for (const attrib of res.object_definition.referencing[ref].attributes) {
          res.object_definition_referencing_by_attribute[attrib] = res.object_definition.referencing[ref].references
          res.object_definition_reference_names_by_attribute[attrib] = ref
        }
      }
      res.object_functions = data.object_functions
      res.object_fq_name = object_fq_name
      vm.$store.commit('cacheDBEditorData', {
        object_fq_name: res.object_fq_name,
        object_definition: res.object_definition,
        object_functions: res.object_functions,
        object_definition_referencing_by_attribute: res.object_definition_referencing_by_attribute,
        object_definition_reference_names_by_attribute: res.object_definition_reference_names_by_attribute
      })
      return res
    }
  }
}
